package opmanager

import (
	"fmt"
	"strings"
)

func getUnitByMonitorUnit(monitor *MonitorDetails) string {
	switch monitor.Unit {
	case "Percentage", "%":
		return "%"

	case "IO Reads", "IO Writes":
		return "iops"

	case "Hours":
		return "h"

	case "Minutes":
		return "m"

	case "Seconds":
		return "s"

	case "Milliseconds":
		return "ms"

	case "B", "Bytes":
		return "B"

	case "KB", "KBytes":
		return "KB"

	case "MB", "MBytes":
		return "MB"

	case "GB", "GBytes":
		return "GB"

	case "bps":
		return "B/s"

	case "Packets per second", "Trasmits (Bps)", "Receives (Bps)":
		return "pps"

	case "Voltage (volts)", "Voltage", "Volts":
		return "V"

	case "Amps":
		return "A"

	}

	// openITCOCKPIT has no unit for "Nos" (Number) or "load"
	return ""
}

func getPerformanceDataAsStringOfMonitor(monitor *MonitorDetails) string {
	var perfdata []string
	for _, metric := range monitor.Data {
		metricName := metric.Instance
		if metricName == "-1" {
			metricName = "value"
		}

		unit := getUnitByMonitorUnit(monitor)
		value, err := metric.Value.Float64()

		if err != nil {
			continue
		}

		if unit == "%" {
			perfdata = append(perfdata, fmt.Sprintf("'%v'=%v%%;;0;100", metricName, value))
		} else {
			perfdata = append(perfdata, fmt.Sprintf("'%v'=%v%v;;;", metricName, value, unit))
		}
	}

	if len(perfdata) > 0 {
		return strings.Join(perfdata[:], " ")
	}

	// No perfdata
	return ""
}
