package opmanager

import jsoniter "github.com/json-iterator/go"

type ErrorDetails struct {
	Message   string `json:"message"`   // "The parameter name is mandatory and must have atleast 1 occurrence(s).",
	Errorcode string `json:"Errorcode"` // LESS_THAN_MIN_OCCURANCE
}

type ResponseError struct {
	Error ErrorDetails `json:"error"`
}

type PaginateResponse[T any] struct {
	Total   jsoniter.Any `json:"total"`   // Total amount of pages
	Page    jsoniter.Any `json:"page"`    // Current page (The listAlarms returns this as string "2" -.- )
	Records jsoniter.Any `json:"records"` // Total amount of records
	Rows    []T          `json:"rows"`    // Generic field which can be a Device or Interface (or any other) go struct
}

type Device struct {
	IsSnmp           bool         `json:"isSNMP,omitempty"`
	Ipaddress        string       `json:"ipaddress,omitempty"`
	StatusStr        string       `json:"statusStr,omitempty"`
	PrettyTime       string       `json:"prettyTime,omitempty"`
	DisplayName      string       `json:"displayName,omitempty"`
	IsNew            bool         `json:"isNew,omitempty"`
	Type             string       `json:"type,omitempty"`       // Linux / MiKroTik RouterOS
	VendorName       string       `json:"vendorName,omitempty"` // MiKroTik
	DeviceName       string       `json:"deviceName,omitempty"` // This is our Identifier
	Moid             int64        `json:"moid,omitempty"`       // 320 - Required for the getLastCollectedValues API endpoint
	StatusNum        jsoniter.Any `json:"statusNum,omitempty"`
	AddedTime        string       `json:"addedTime,omitempty"`
	InterfaceCount   int64        `json:"interfaceCount,omitempty"`
	Category         string       `json:"category,omitempty"`         // Server / Switch
	ProbeName        string       `json:"probeName,omitempty"`        // Only on Enterprise Systems e.g.: MERAKI_PROBE
	ProbeDisplayName string       `json:"probeDisplayName,omitempty"` // Only on Enterprise Systems of OpManager e.g.: IS_MERAKI
	Monitors         []Monitor    `json:"monitors,omitempty"`         // Monitors of a Device (Service checks)
}

type Interface struct {
	Id                   string `json:"id"`                   // 1 - We dont know what this Id is about !
	InterfaceDisplayName string `json:"interfaceDisplayName"` // "eth0-eth0"
	StatusStr            string `json:"statusStr"`            // Clear / Critical ...
	DisplayName          string `json:"displayName"`          // Name of the Host
	IfName               string `json:"ifName"`               // eth0
	IpAddress            string `json:"ipAddress"`            // 192.168.1.1
	OutSpeed             string `json:"outSpeed"`             //"0 bps"
	InSpeed              string `json:"inSpeed"`              //"0 bps"
	Iftype               string `json:"type,omitempty"`       // Ethernet, Software Loopback
	DeviceName           string `json:"deviceName"`           // Identifier of the Host
	InterfaceStatus      int64  `json:"interfaceStatus"`      // 5
	Moid                 int64  `json:"moid"`                 // 320 ¯\_(ツ)_/¯
	OutTraffic           string `json:"outTraffic"`           // Tx Traffic - "16.137 K (0.0%)"
	StatusNum            int64  `json:"statusNum"`            // 5
	InTraffic            string `json:"inTraffic"`            // Rx Traffic - "938.505  (0.0%)"
	AdminStatus          string `json:"adminStatus"`          // Up
	OperStatus           string `json:"operStatus"`           // Up
	PhysAddr             string `json:"physAddr"`             // da fe fb 47 d0 0e
	InterfaceName        string `json:"interfaceName"`        // IF-opmanager.itsm.love-320 // Identifier of the Service
}

type Alert struct {
	StatusStr   string `json:"statusStr"`   // "Critical"
	PrettyTime  string `json:"prettyTime"`  // "7 days ago"
	DisplayName string `json:"displayName"` // "172.16.166.46"
	// IpAddress   string `json:"ipAddress"`   // "172.16.166.46" - gone since OpManager version "12.8.270"
	Technician string `json:"technician"` // "admin"
	Message    string `json:"message"`    // "Device Down: No response from device for last 5 polls"
	DeviceName string `json:"deviceName"` // "172.16.166.46" - Identifier of the Host
	StatusNum  int64  `json:"statusNum"`  // 1
	ModTime    string `json:"modTime"`    // "17 Nov 2023 01:31:02 PM CET"
	AlarmId    int64  `json:"alarmId"`    // 1
	Eventtype  string `json:"eventtype"`  // "Device Down"
	Category   string `json:"category"`   // "Server"
	Entity     string `json:"entity"`     // "77_RAMUSAGE" "172.16.166.46_Poll"
	Who        string `json:"who"`        // "admin"
}

type Monitor struct {
	Protocol        string `json:"protocol"`        // SNMP
	MonitorType     string `json:"monitorType"`     // "performanceMonitors"
	StatusStr       string `json:"statusStr"`       // Trouble
	MonitorName     string `json:"monitorName"`     // "Win-MemoryUtilization"
	DisplayName     string `json:"displayName"`     // "Opmanager"
	PollID          string `json:"pollID"`          // "77" - ID we use as Service Identifier for Monitor Services in openITCOCKPIT. Also this will be used to map OpManager Alerts to Monitors
	MonitorDispName string `json:"monitorDispName"` // "Memory Utilization"
	DeviceName      string `json:"deviceName"`      // "opmanager.itsm.love" - Identifier of the Host
	MonSeverity     string `json:"monSeverity"`     // "2" - State-Code as string (OPMANAGER_STATUS_TROUBLE, OPMANAGER_STATUS_CLEAR etc)
}

type DeviceAssociatedMonitors struct {
	PerformanceMonitors PerformanceMonitors `json:"performanceMonitors"`
}

type PerformanceMonitors struct {
	Total    int64            `json:"total"`
	Down     int64            `json:"down"`
	Monitors []MonitorDetails `json:"monitors"`
}

type MonitorDetails struct {
	Unit              string              `json:"YAXISTEXT"`         // "%"" or "IO Reads" or "Percentage" or <whatever> - Unit of the Metric
	ThresholdEnabled  string              `json:"THRESHOLDENABLED"`  // "true" or "false"
	PollId            string              `json:"pollId"`            // 77 - - Identifier of the Service
	Protocol          string              `json:"Protocol"`          // NCM / SNMP
	DisplayName       string              `json:"DISPLAYNAME"`       // Config change Count - Fancy name of the service
	Active            string              `json:"active"`            // "true" - UNUSED -
	ThresholdStatus   string              `json:"thresholdStatus"`   // "Not Enabled", "Normal", "Violated" - UNUSED -
	PollingStatus     string              `json:"pollingStatus"`     // 0 = Active, 2 = Suspended, 15 = Disabled (more values possible?)
	LastCollectedTime string              `json:"lastCollectedTime"` // "27 Nov 2023 05:14:59 PM CET"
	Id                string              `json:"Id"`                // 77 (same as PollId ?) - UNUSED -
	Interval          string              `json:"interval"`          // Check interval in minutes
	Data              []MonitorMetricData `json:"data"`              // Metrics / Gauges of the Monitor
	MonSeverity       string              // "2" - State-Code as string (OPMANAGER_STATUS_TROUBLE, OPMANAGER_STATUS_CLEAR etc) - WE copy the value of this over manually because the API is 💩
}

type MonitorMetricData struct {
	Instance string          `json:"instance"` // Name of the metric
	Value    jsoniter.Number `json:"value"`    // Current value of the metric
}

type Probe struct {
	DisplayName string `json:"DISPLAYNAME"` // IS_MERAKI
	Status      bool   `json:"STATUS"`      // Unknown
	ProbeId     string `json:"PROBEID"`     // "40000000001"
	IsWindows   bool   `json:"ISWINDOWS"`   // false
	IpAddress   string `json:"IPADDRESS"`   // 10.206.162.115
	SiteId      string `json:"SITEID"`      // "303"
	Name        string `json:"NAME"`        // MERAKI_PROBE
}

// LastCollectedValues is the response of the getLastCollectedValues API endpoint
// All uncommented fields are not used in openITCOCKPIT, for now
type LastCollectedValues struct {
	//NumericType       int64               `json:"NumericType"`       // 1
	//Agent             string              `json:"Agent"`             // "mbdmlb110.mn-man.biz.20000000001"
	PollingStatus string `json:"PollingStatus"` // "0"
	ProtocolName  string `json:"ProtocolName"`  // "SNMP"
	Description   string `json:"Description"`   // "Monitors the Memory utilization percentage."
	//GraphName         string              `json:"GraphName"`         // "NSMemoryUtilizationn"
	//GraphTypeName     string              `json:"GraphTypeName"`     // "node"
	Data []MonitorMetricData `json:"data"` // Metrics / Gauges of the Monitor
	//GraphId           int64               `json:"GraphId"`           // 146
	PollId            int64  `json:"PollId"`            // 20000000127
	ThresholdEnabled  bool   `json:"ThresholdEnabled"`  // true
	Active            bool   `json:"active"`            // true
	YAxisText         string `json:"YAxisText"`         // "Percentage"
	LastCollectedTime string `json:"lastCollectedTime"` // "3 Jul 2025 03:25:37 PM CEST"
	DisplayName       string `json:"DisplayName"`       // "Memory Utilization"
	Interval          int64  `json:"Interval"`          // 5
}
